// --------------------------------------------------------------------
// Author  : mashimonator
// Create  : 2010/03/17
// Update  : 2010/03/17
//         : 2010/03/19 ページ印刷に対応
// Description : 画像を遅延ロードする
// --------------------------------------------------------------------

/*@cc_on 
var doc = document;
eval('var document = doc');
@*/
var lazyload = {
	//-----------------------------------------
	// 設定値
	//-----------------------------------------
	config : {
		cls : '', // クラス指定する場合のクラス名（未指定の場合は全画像が対象）
		blank : 'images/blank.gif', // blank画像のパス
		outside : true, // ファーストビュー時に画面外にある画像のみを対象とする
		fade : true, // 画像をフェードインする
		speed : 50, // フェードインの実行間隔（ミリ秒単位）
		step : 0.5 // フェードインの加算値
	},
	storage : {
		screenHeight : null,
		imgs : []
	},
	//-----------------------------------------
	// 初期処理
	//-----------------------------------------
	initialize : function() {
		// 現在の画面表示位置を取得
		var currentTop = window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0;
		// 画面サイズを保持
		var size = lazyload.getScreenSize();
		lazyload.storage.screenHeight = size.height;
		// img要素を取得
		var imgs = lazyload.getTargetElements('IMG', lazyload.config.cls);
		for (var i = 0, len = imgs.length; i < len; i++) {
			// 要素の位置を取得
			var pos = lazyload.getElementPosition(imgs[i]);
			// 遅延ロード
			if ( lazyload.config.outside ) {
				// 画面外の要素が対象の場合
				if ( pos.top > (currentTop + lazyload.storage.screenHeight) ) {
					lazyload.setLazy(imgs[i], pos.top, true);
				}
			} else {
				// 画面内の要素も対象の場合
				if ( pos.top > (currentTop + lazyload.storage.screenHeight) ) {
					lazyload.setLazy(imgs[i], pos.top, true);
				} else {
					lazyload.setLazy(imgs[i], pos.top, false);
				}
			}
		}
		// 画像プリロード
		lazyload.addEvent(window, 'load', lazyload.preLoadImages);
		// スクロールイベントをセット
		lazyload.addEvent(window, 'scroll', lazyload.scroll);
	},
	//-----------------------------------------
	// 遅延ロード
	//-----------------------------------------
	setLazy : function( img, top, outside ) {
		// 元の画像パスを保持
		img.orgSrc = img.src;
		// blank画像に置き換え
		img.src = lazyload.config.blank;
		// 要素を保持
		lazyload.storage.imgs[lazyload.storage.imgs.length] = {'img' : img, 'top' : top, 'outside' : outside, 'executed' : false};
		// 透明度をセット
		lazyload.setOpacity(img, 0);
	},
	//-----------------------------------------
	// スクロールイベント
	//-----------------------------------------
	scroll : function() {
		// 現在の画面表示位置を取得
		var currentTop = window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0;
		// 非表示（遅延ロード対象）画像を取得
		var imgs = lazyload.storage.imgs;
		for (var i = 0, len = imgs.length; i < len; i++) {
			// 画面内の要素を対象に処理
			if ( ((currentTop + lazyload.storage.screenHeight) > imgs[i].top) && !imgs[i].executed ) {
				// 表示済みフラグをセット
				lazyload.storage.imgs[i].executed = true;
				// 画像表示
				lazyload.show(imgs[i].img);
			}
		}
	},
	//-----------------------------------------
	// 画像表示
	//-----------------------------------------
	show : function(img) {
		// fade用関数
		var fadeIn = function(img){
			return function(){
				// 不透明度取得
				var opacity = lazyload.getOpacity(img);
				if ( opacity < 10 ) {
					// 透過処理
					lazyload.setOpacity(img, (opacity + lazyload.config.step));
					// 再帰呼び出し
					window.setTimeout(fadeIn, lazyload.config.speed);
				}
			}
		}(img);
		// 表示
		if ( lazyload.config.fade ) {
			// フェードイン有効
			fadeIn();
		} else {
			// フェードイン無効
			lazyload.setOpacity(img, 10);
		}
	},
	//-----------------------------------------
	// 画像プリロード
	//-----------------------------------------
	preLoadImages : function() {
		// 対象要素取得
		var imgs = lazyload.storage.imgs;
		for (var i = 0, len = imgs.length; i < len; i++) {
			// プリロード
			var d = document;
			d.preLoad = new Image();
			d.preLoad.src = imgs[i].img.orgSrc;
			// 印刷用クラスセット
			if ( !lazyload.config.cls || lazyload.config.cls == '' ) {
				imgs[i].img.className += ' lazyLoadPrint';
			}
			// 元画像セット
			imgs[i].img.src = imgs[i].img.orgSrc;
			// 画面内の要素も処理対象＆画面内の要素の場合
			if ( !lazyload.config.outside && !imgs[i].outside ) {
				// 表示済みフラグをセット
				lazyload.storage.imgs[i].executed = true;
				// 画像表示
				lazyload.show(imgs[i].img);
			}
		}
		// 印刷用スタイルセット
		if ( !lazyload.config.cls || lazyload.config.cls == '' ) {
			lazyload.addPrintCssRule('.lazyLoadPrint', 'filter:alpha(opacity=100)!important; opacity:1.0!important;');
		} else {
			lazyload.addPrintCssRule('.' + lazyload.config.cls, 'filter:alpha(opacity=100)!important; opacity:1.0!important;');
		}
	},
	//-----------------------------------------
	// スタイルシートに規則を追加する
	//-----------------------------------------
	addPrintCssRule : function ( selector, declarations ) {
		var style = document.createElement('style');
		style.setAttribute('type', 'text/css');
		style.setAttribute('media', 'print');
		document.getElementsByTagName('head')[0].appendChild(style);
		if ( lazyload.isIE() ) {
			style.styleSheet.addRule( selector, declarations );
		} else {
			style.sheet.insertRule(selector + '{' + declarations + '}', 0);
		}
	},
	//-----------------------------------------
	// IE判定
	//-----------------------------------------
	isIE : function() {
		return IE='\v'=='v';
	},
	//-----------------------------------------
	// 要素の透明度を取得する
	//-----------------------------------------
	getOpacity : function( element ) {
		if ( element.alpha ) {
				return element.alpha / 10;
		} else if ( element.style ) {
			if ( element.style.opacity ) {
				return element.style.opacity * 10;
			}
		} 
		return 0;
	},
	//-----------------------------------------
	// 要素の透明度を設定する
	//-----------------------------------------
	setOpacity : function( element, opacity ) {
		element.style.opacity = opacity / 10;
		element.style.filter = 'alpha(opacity=' + opacity * 10 + ')';
		element.alpha = opacity * 10;
	},
	//-----------------------------------------
	// ブラウザの画面サイズを取得する
	//-----------------------------------------
	getScreenSize : function() {
		var w = 0;
		var h = 0;
		if ( window.innerWidth ) {
			w = window.innerWidth;
		} else if ( document.documentElement && document.documentElement.clientWidth != 0 ) {
			w = document.documentElement.clientWidth;
		} else if ( document.body ) {
			w = document.body.clientWidth;
		}
		if ( window.innerHeight ) {
			h = window.innerHeight;
		} else if ( document.documentElement && document.documentElement.clientHeight != 0 ) {
			h = document.documentElement.clientHeight;
		} else if ( document.body ) {
			h = document.body.clientHeight;
		}
		return ({ 'width': w, 'height': h });
	},
	//-----------------------------------------
	// 対象要素のポジションを取得する
	//-----------------------------------------
	getElementPosition : function( element ) {
		var offsetTrail = (typeof element == 'string') ? document.getElementById(element): element;
		var x = 0;
		var y = 0;
		while (offsetTrail) {
			x += offsetTrail.offsetLeft;
			y += offsetTrail.offsetTop;
			offsetTrail = offsetTrail.offsetParent;
		}
		if ( navigator.userAgent.indexOf('Mac') != -1 && typeof document.body.leftMargin != 'undefined' ) {
			x += document.body.leftMargin;
			y += document.body.topMargin;
		}
		return ({ 'left': x, 'top': y });
	},
	//-----------------------------------------
	// ターゲットタグを取得する
	//-----------------------------------------
	getTargetElements : function( tag, cls ) {
		var elements = new Array();
		var targetElements = document.getElementsByTagName(tag.toUpperCase());
		if ( !cls || cls == '' ) {
			elements = targetElements;
		} else {
			for ( var i = 0, len = targetElements.length; i < len; i++ ) {
				if ( targetElements[i].className.match(cls) ) {
					elements[elements.length] = targetElements[i];
				}
			}
		}
		return elements;
	},
	//-----------------------------------------
	// イベントに関数を付加する
	//-----------------------------------------
	addEvent : function( target, event, func ) {
		try {
			target.addEventListener(event, func, false);
		} catch (e) {
			target.attachEvent('on' + event, (function(el){return function(){func.call(el);};})(target));
		}
	}
}
var clListener = {
	isAlreadyDetect : false,
	isAlreadyExec : false,
	funcList : [],
	//-----------------------------------------
	// DOMContentLoadedイベントで実行する関数を登録する
	//-----------------------------------------
	add : function( func ) {
		if ( clListener.isAlreadyExec ) {
			func.call();
		} else {
			clListener.funcList.push( function(){ return func.call(); } );
		}
	},
	//-----------------------------------------
	// DOMContentLoadedイベントを検知する
	//-----------------------------------------
	detectEvent : function() {
		// check
		if ( clListener.isAlreadyDetect ) {
			return;
		} else {
			clListener.isAlreadyDetect = true;
		}
		// detect
		if ( document.addEventListener ) {
			// for Mozilla, Opera and webkit
			document.addEventListener('DOMContentLoaded', function(){
				document.removeEventListener('DOMContentLoaded', arguments.callee, false);
				clListener.exec();
			}, false);
		} else if ( document.attachEvent ) {
			// for IE(iframe)
			document.attachEvent('onreadystatechange', function(){
				if ( document.readyState === 'complete' ) {
					document.detachEvent('onreadystatechange', arguments.callee);
					clListener.exec();
				}
			});
			// for IE
			if ( document.documentElement.doScroll && window == window.top ) (function(){
				if ( clListener.isAlreadyExec ) return;
				try {
					document.documentElement.doScroll('left');
				} catch( error ) {
					setTimeout(arguments.callee, 0);
					return;
				}
				clListener.exec();
			})();
		}
	},
	//-----------------------------------------
	// 登録された関数を実行する
	//-----------------------------------------
	exec : function() {
		if ( !clListener.isAlreadyExec ) {
			clListener.isAlreadyExec = true;
			if ( clListener.funcList ) {
				for (var i = 0, len = clListener.funcList.length; i < len; i++) {
					clListener.funcList[i].apply();
				}
				clListener.funcList = null;
			}
		}
	}
}
clListener.detectEvent();
clListener.add(lazyload.initialize);